// smart_assert_ctxfunc.hpp

// Boost.SmartAssert library
//
// Copyright (C) 2003 John Torjo (john@torjo.com)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
 
// See http://www.boost.org for updates, documentation, and revision history.

#ifndef BOOST_INNER_SMART_ASSERT_CONTEXT_FUNC_HPP_INCLUDED
#define BOOST_INNER_SMART_ASSERT_CONTEXT_FUNC_HPP_INCLUDED

#if _MSC_VER >= 1020
#pragma once
#endif


#include "smart_assert_l_basic.hpp"
#include "smart_assert_context.hpp"

// FIXME - make this easier to use.

namespace boost { 

namespace smart_assert {

// forward declaration(s)
class assert_context;

namespace Private {


    // note: we created our own assert_context wrapper,
    // because #including <boost/function.hpp> would take too long!!!
    //
    // besides, VC6 gives an INTERNAL COMPILER ERROR

    struct assert_context_func_impl_base {

        virtual void call_func( const assert_context &context) const = 0;
        virtual assert_context_func_impl_base* clone() const = 0;
    };

    template< class type>
    struct assert_context_func_impl : public assert_context_func_impl_base {

        assert_context_func_impl() : m_f() {
        }
        assert_context_func_impl( type f) : m_f( f) {
        }
        void call_func( const assert_context &context) const {
            m_f( context);
        }
        assert_context_func_impl_base* clone() const {
            return new assert_context_func_impl< type>( *this);
        }
    private:
        mutable type m_f;
    };

} // namespace Private 


// functor taking an assert_context argument
//
struct assert_context_func {

    typedef void (*func)( const assert_context&);

    assert_context_func( func f = 0) : m_pimpl( 0) {
        if ( f)
            m_pimpl = new Private::assert_context_func_impl< func>(f);
    }

    assert_context_func( const assert_context_func & val) 
        : m_pimpl( get_clone( val.m_pimpl) ) {
    }

    /*
    FAILED on VC6 (when compiling std::pair< int, assert_context_func>

    template< class type>
    assert_context_func( type val ) 
        : m_pimpl( new assert_context_func_impl< type>(val) ) {
    }
    */


    ~assert_context_func() {
        delete m_pimpl;
    }

    void operator() ( const assert_context &context) const {
        if ( m_pimpl)
            m_pimpl->call_func( context);
    }

    void copy( const assert_context_func & val) {
        delete m_pimpl;
        m_pimpl = get_clone( val.m_pimpl);
    }

    template< class T> 
        assert_context_func & assign( const T & val) {
    
        delete m_pimpl;
        m_pimpl = new Private::assert_context_func_impl< T>(val);
        return *this;
    }

    assert_context_func & assign( func f) {

        delete m_pimpl;
        m_pimpl = new Private::assert_context_func_impl< func>(f);
        return *this;
    }

    void operator=( const assert_context_func & val) {
        copy( val);
    }

private:
    static Private::assert_context_func_impl_base * get_clone( Private::assert_context_func_impl_base * pimpl) {
        return pimpl ? pimpl->clone() : 0;
    }


private:
    Private::assert_context_func_impl_base * m_pimpl;
};


} // namespace smart_assert

} // namespace boost


#endif
