// smart_assert_print.hpp

// Boost.SmartAssert library
//
// Copyright (C) 2003 John Torjo (john@torjo.com)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
 
// See http://www.boost.org for updates, documentation, and revision history.

#ifndef BOOST_INNER_SMART_ASSERT_PRINT_HPP_INCLUDED
#define BOOST_INNER_SMART_ASSERT_PRINT_HPP_INCLUDED

#if _MSC_VER >= 1020
#pragma once
#endif

// classes for printing the variables that
// are involved in an ASSERT, in case the ASSERT fails

#include "smart_assert_context.hpp"
#include <iostream>

namespace boost { 

namespace smart_assert {

/* 
    Allows printing a range

  Example:

  ASSERT( *first != *(last-1) )(range_(first,last));

  Will print out the values of first, last, and the range
  [first, last)
*/
template< class iterator_type>
struct printer_range_t {
    printer_range_t( iterator_type first, iterator_type last)
        : m_first( first), m_last( last) {
    }

    // note: we treat the iterator_type as dereferenceable!!!
    iterator_type m_first, m_last;
};

template< class iterator_type>
inline client_ostream_type & operator<<( client_ostream_type & out, const printer_range_t< iterator_type> &val) {
    out << "Range= [" << &*(val.m_first) << "," << &*( val.m_last) << "); "
        << "Values= [";
    iterator_type first = val.m_first;
    while ( first != val.m_last) {
        if ( first != val.m_first)
            out << ", ";
        out << "'" << *first << "'";
        ++first;
    }
    out << "]";
    return out;
}

template< class iterator_type>
inline printer_range_t< iterator_type> range_( iterator_type first, iterator_type last) {
    return printer_range_t< iterator_type>( first, last);
}


struct default_printer {

    template< class type>
    int print_val( assert_context & context, const type & val, const char_type * str) {
        ostringstream_type out;
        out << val;
        
        return context.add_val( out.str(), str);
    }
};

} // namespace smart_assert

} // namespace boost


#endif
